@@ -4,10 +4,10 @@
 # a single simput catalog from the command line.
 
 # First, make an absorbed thermal spectrum, which will be for an annulus source.
-make_thermal_spectrum 6.0 0.3 0.05 1.0e-4 thermal_spec.dat 0.1 10.0 10000 --absorb_model=wabs --nh 0.04 --overwrite
+make_cie_spectrum 6.0 0.3 0.05 1.0e-4 thermal_spec.dat 0.1 10.0 10000 --absorb_model=wabs --nH_abs 0.04 --overwrite
 
 # Second, make an absorbed power-law spectrum, which will be for a point source.
-make_powerlaw_spectrum 1.1 0.05 1.0e-4 powerlaw_spec.dat 0.1 10.0 10000 --absorb_model=wabs --nh 0.04 --overwrite
+make_powerlaw_spectrum 1.1 0.05 1.0e-4 powerlaw_spec.dat 0.1 10.0 10000 --absorb_model=wabs --nH_abs 0.04 --overwrite
 
 # Take this spectrum and make a SIMPUT catalog with a point source photon list
 make_point_source my_cat.simput point_source 30.0 45.0 thermal_spec.dat --overwrite
