@@ -37,6 +37,9 @@ def write_event_file(events, parameters, filename, overwrite=False):
     col_id = fits.Column(
         name="CCD_ID", format="J", unit="pixel", array=events["ccd_id"]
     )
+    col_se = fits.Column(
+        name="SOXS_ENERGY", format="E", unit="eV", array=events["soxs_energy"] * 1000.0
+    )
 
     chantype = parameters["channel_type"].upper()
     if chantype == "PHA":
@@ -47,7 +50,7 @@ def write_event_file(events, parameters, filename, overwrite=False):
 
     col_t = fits.Column(name="TIME", format="1D", unit="s", array=events["time"])
 
-    cols = [col_e, col_x, col_y, col_ch, col_t, col_dx, col_dy, col_id]
+    cols = [col_e, col_x, col_y, col_ch, col_t, col_dx, col_dy, col_id, col_se]
 
     coldefs = fits.ColDefs(cols)
     tbhdu = fits.BinTableHDU.from_columns(coldefs)
@@ -356,7 +359,14 @@ def make_exposure_map(
 
 
 def _write_spectrum(
-    bins, spec, exp_time, spectype, parameters, specfile, overwrite=False
+    bins,
+    spec,
+    exp_time,
+    spectype,
+    parameters,
+    specfile,
+    overwrite=False,
+    noisy=True,
 ):
 
     col1 = fits.Column(name="CHANNEL", format="1J", array=bins)
@@ -384,7 +394,7 @@ def _write_spectrum(
     tbhdu.header["CHANTYPE"] = spectype
     tbhdu.header["BACKFILE"] = "none"
     tbhdu.header["CORRFILE"] = "none"
-    tbhdu.header["POISSERR"] = True
+    tbhdu.header["POISSERR"] = noisy
     for key in ["RESPFILE", "ANCRFILE", "MISSION", "TELESCOP", "INSTRUME"]:
         tbhdu.header[key] = parameters[key]
     tbhdu.header["AREASCAL"] = 1.0
@@ -396,30 +406,45 @@ def _write_spectrum(
     hdulist.writeto(specfile, overwrite=overwrite)
 
 
-def _region_filter(hdu, region, format="ds9"):
+def _region_filter(hdu, region, format="ds9", exclude=False):
     from regions import PixCoord, PixelRegion, Region, Regions, SkyRegion
 
     if isinstance(region, str):
         if Path(region).exists():
-            region = Regions.read(region, format=format)[0]
+            region = Regions.read(region, format=format)
         else:
-            region = Regions.parse(region, format=format)[0]
-    elif not isinstance(region, Region):
+            region = Regions.parse(region, format=format)
+    elif not isinstance(region, (Region, Regions)):
         raise RuntimeError("'region' argument is not valid!")
     pixcoords = PixCoord(hdu.data["X"], hdu.data["Y"])
-    if isinstance(region, PixelRegion):
-        evt_mask = region.contains(pixcoords)
-    elif isinstance(region, SkyRegion):
-        w = wcs_from_header(hdu.header)
-        skycoords = pixcoords.to_sky(w, origin=1)
-        evt_mask = region.contains(skycoords, w)
-    else:
-        raise NotImplementedError
+    if isinstance(region, Region):
+        region = [region]
+    evt_mask = False
+    for r in region:
+        if isinstance(r, PixelRegion):
+            evt_mask |= r.contains(pixcoords)
+        elif isinstance(r, SkyRegion):
+            w = wcs_from_header(hdu.header)
+            skycoords = pixcoords.to_sky(w, origin=1)
+            evt_mask |= r.contains(skycoords, w)
+        else:
+            raise NotImplementedError
+    if exclude:
+        evt_mask = ~evt_mask
     return evt_mask
 
 
 def filter_events(
-    evtfile, newfile, region=None, emin=None, emax=None, format="ds9", overwrite=False
+    evtfile,
+    newfile,
+    region=None,
+    emin=None,
+    emax=None,
+    tmin=None,
+    tmax=None,
+    format="ds9",
+    exclude=False,
+    overwrite=False,
 ):
     r"""
 
@@ -432,14 +457,23 @@ def filter_events(
     region : string or Region, optional
         The region to be used for the filtering. Default: None
     emin : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
-        The minimum energy of the events to be binned in keV.
+        The minimum energy of the events to be included, in keV.
         Default is the lowest energy available.
     emax : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
-        The maximum energy of the events to be binned in keV.
+        The maximum energy of the events to be included, in keV.
         Default is the highest energy available.
+    tmin : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
+        The minimum energy of the events to be included, in seconds.
+        Default is the earliest time available.
+    tmax : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
+        The maximum energy of the events to be included, in seconds.
+        Default is the latest time available.
     format : string, optional
-        The file format specifier for the region. "ds9",
+        The file format specifier for the region, if supplied. "ds9",
         "crtf", "fits", etc. Default: "ds9"
+    exclude : boolean, optional
+        If True, the events in a specified *region* will be excluded instead of
+        included. Default: False
     overwrite : boolean, optional
         Whether to overwrite an existing file with the
         same name. Default: False
@@ -448,18 +482,35 @@ def filter_events(
         hdu = f["EVENTS"]
         evt_mask = np.ones(hdu.data["ENERGY"].size, dtype="bool")
         if region is not None:
-            evt_mask &= _region_filter(hdu, region, format=format)
+            evt_mask &= _region_filter(hdu, region, format=format, exclude=exclude)
         if emin is not None:
             emin = parse_value(emin, "keV") * 1000.0
             evt_mask &= hdu.data["ENERGY"] > emin
         if emax is not None:
             emax = parse_value(emax, "keV") * 1000.0
             evt_mask &= hdu.data["ENERGY"] < emax
+        if tmin is not None:
+            tmin = parse_value(tmin, "s")
+            evt_mask &= hdu.data["TIME"] > tmin
+        if tmax is not None:
+            tmax = parse_value(tmax, "s")
+            evt_mask &= hdu.data["TIME"] < tmax
         hdu.data = hdu.data[evt_mask]
         f.writeto(newfile, overwrite=overwrite)
 
 
-def write_spectrum(evtfile, specfile, region=None, format="ds9", overwrite=False):
+def write_spectrum(
+    evtfile,
+    specfile,
+    region=None,
+    format="ds9",
+    exclude=False,
+    emin=None,
+    emax=None,
+    tmin=None,
+    tmax=None,
+    overwrite=False,
+):
     r"""
     Bin event energies into a spectrum and write it to
     a FITS binary table. Does not do any grouping of
@@ -476,6 +527,21 @@ def write_spectrum(evtfile, specfile, region=None, format="ds9", overwrite=False
     format : string, optional
         The file format specifier for the region. "ds9",
         "crtf", "fits", etc. Default: "ds9"
+    exclude : boolean, optional
+        If True, the events in a specified *region* will be excluded
+        instead of included in the spectrum. Default: False
+    emin : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
+        The minimum energy of the events to be included, in keV.
+        Default is the lowest energy available.
+    emax : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
+        The maximum energy of the events to be included, in keV.
+        Default is the highest energy available.
+    tmin : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
+        The minimum energy of the events to be included, in seconds.
+        Default is the earliest time available.
+    tmax : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
+        The maximum energy of the events to be included, in seconds.
+        Default is the latest time available.
     overwrite : boolean, optional
         Whether to overwrite an existing file with
         the same name. Default: False
@@ -488,7 +554,19 @@ def write_spectrum(evtfile, specfile, region=None, format="ds9", overwrite=False
             hdu = f["EVENTS"]
             evt_mask = np.ones(hdu.data["ENERGY"].size, dtype="bool")
             if region is not None:
-                evt_mask &= _region_filter(hdu, region, format=format)
+                evt_mask &= _region_filter(hdu, region, format=format, exclude=exclude)
+            if tmin is not None:
+                tmin = parse_value(tmin, "s")
+                evt_mask &= hdu.data["TIME"] > tmin
+            if tmax is not None:
+                tmax = parse_value(tmax, "s")
+                evt_mask &= hdu.data["TIME"] < tmax
+            if emin is not None:
+                emin = parse_value(emin, "keV") * 1000.0
+                evt_mask &= hdu.data["ENERGY"] > emin
+            if emax is not None:
+                emax = parse_value(emax, "keV") * 1000.0
+                evt_mask &= hdu.data["ENERGY"] < emax
             spectype = hdu.header["CHANTYPE"]
             rmf = hdu.header["RESPFILE"]
             p = hdu.data[spectype][evt_mask]
@@ -680,6 +758,8 @@ def write_image(
     coord_type="sky",
     emin=None,
     emax=None,
+    tmin=None,
+    tmax=None,
     overwrite=False,
     expmap_file=None,
     reblock=1,
@@ -701,6 +781,12 @@ def write_image(
         The minimum energy of the photons to put in the image, in keV.
     emax : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
         The maximum energy of the photons to put in the image, in keV.
+    tmin : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
+        The minimum energy of the events to be included, in seconds.
+        Default is the earliest time available.
+    tmax : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, optional
+        The maximum energy of the events to be included, in seconds.
+        Default is the latest time available.
     overwrite : boolean, optional
         Whether to overwrite an existing file with
         the same name. Default: False
@@ -709,7 +795,7 @@ def write_image(
         to get a flux map. Default: None
     reblock : integer, optional
         Change this value to reblock the image to larger
-        pixel sizes (reblock >= 1). Only supported for
+        or small pixel sizes. Only supported for
         sky coordinates. Default: 1
     """
     if emin is None:
@@ -722,30 +808,39 @@ def write_image(
     else:
         emax = parse_value(emax, "keV")
     emax *= 1000.0
-    if coord_type == "det" and reblock > 1:
+    if tmin is None:
+        tmin = -np.inf
+    else:
+        tmin = parse_value(tmin, "s")
+    if tmax is None:
+        tmax = np.inf
+    else:
+        tmax = parse_value(tmax, "s")
+    if coord_type == "det" and reblock != 1:
         raise RuntimeError(
-            "Reblocking images is not supported " "for detector coordinates!"
+            "Reblocking images is not supported for detector coordinates!"
         )
-    f = fits.open(evt_file)
-    e = f["EVENTS"].data["ENERGY"]
-    idxs = np.logical_and(e > emin, e < emax)
-    xcoord, ycoord, xcol, ycol = coord_types[coord_type]
-    x = f["EVENTS"].data[xcoord][idxs]
-    y = f["EVENTS"].data[ycoord][idxs]
-    exp_time = f["EVENTS"].header["EXPOSURE"]
-    xmin = f["EVENTS"].header[f"TLMIN{xcol}"]
-    ymin = f["EVENTS"].header[f"TLMIN{ycol}"]
-    xmax = f["EVENTS"].header[f"TLMAX{xcol}"]
-    ymax = f["EVENTS"].header[f"TLMAX{ycol}"]
-    if coord_type == "sky":
-        xctr = f["EVENTS"].header[f"TCRVL{xcol}"]
-        yctr = f["EVENTS"].header[f"TCRVL{ycol}"]
-        xdel = f["EVENTS"].header[f"TCDLT{xcol}"] * reblock
-        ydel = f["EVENTS"].header[f"TCDLT{ycol}"] * reblock
-    f.close()
+    with fits.open(evt_file) as f:
+        e = f["EVENTS"].data["ENERGY"]
+        t = f["EVENTS"].data["TIME"]
+        idxs = np.logical_and(e > emin, e < emax)
+        idxs &= np.logical_and(t > tmin, t < tmax)
+        xcoord, ycoord, xcol, ycol = coord_types[coord_type]
+        x = f["EVENTS"].data[xcoord][idxs]
+        y = f["EVENTS"].data[ycoord][idxs]
+        exp_time = f["EVENTS"].header["EXPOSURE"]
+        xmin = f["EVENTS"].header[f"TLMIN{xcol}"]
+        ymin = f["EVENTS"].header[f"TLMIN{ycol}"]
+        xmax = f["EVENTS"].header[f"TLMAX{xcol}"]
+        ymax = f["EVENTS"].header[f"TLMAX{ycol}"]
+        if coord_type == "sky":
+            xctr = f["EVENTS"].header[f"TCRVL{xcol}"]
+            yctr = f["EVENTS"].header[f"TCRVL{ycol}"]
+            xdel = f["EVENTS"].header[f"TCDLT{xcol}"] * reblock
+            ydel = f["EVENTS"].header[f"TCDLT{ycol}"] * reblock
 
-    nx = int(xmax - xmin) // reblock
-    ny = int(ymax - ymin) // reblock
+    nx = int(int(xmax - xmin) // reblock)
+    ny = int(int(ymax - ymin) // reblock)
 
     xbins = np.linspace(xmin, xmax, nx + 1, endpoint=True)
     ybins = np.linspace(ymin, ymax, ny + 1, endpoint=True)
@@ -758,15 +853,16 @@ def write_image(
                 "Cannot divide by an exposure map for images "
                 "binned in detector coordinates!"
             )
-        f = fits.open(expmap_file)
-        if f["EXPMAP"].shape != (nx, ny):
-            raise RuntimeError("Exposure map and image do not have the same shape!!")
-        with np.errstate(invalid="ignore", divide="ignore"):
-            H /= f["EXPMAP"].data.T
-        H[np.isinf(H)] = 0.0
-        H = np.nan_to_num(H)
-        H[H < 0.0] = 0.0
-        f.close()
+        with fits.open(expmap_file) as f:
+            if f["EXPMAP"].shape != (nx, ny):
+                raise RuntimeError(
+                    "Exposure map and image do not have the same shape!!"
+                )
+            with np.errstate(invalid="ignore", divide="ignore"):
+                H /= f["EXPMAP"].data.T
+            H[np.isinf(H)] = 0.0
+            H = np.nan_to_num(H)
+            H[H < 0.0] = 0.0
 
     hdu = fits.PrimaryHDU(H.T)
 
@@ -859,7 +955,8 @@ def plot_spectrum(
         If set to True, the counts instead of the count rate will
         be plotted. Default: False
     noerr : boolean, optional
-        If True, the spectrum will be plotted without errorbars.
+        If True, the spectrum will be plotted without errorbars. This
+        will always be the case if the spectrum is not noisy.
         Default: False
     plot_used : boolean, optional
         If set to True, only the bins which contain more than 0
@@ -878,6 +975,8 @@ def plot_spectrum(
     hdu = f["SPECTRUM"]
     chantype = hdu.header["CHANTYPE"]
     y = hdu.data["COUNTS"].astype("float64")
+    if not hdu.header["POISSERR"]:
+        noerr = True
     if plot_energy:
         rmf = hdu.header.get("RESPFILE", None)
         if rmf is not None:
@@ -1037,3 +1136,187 @@ def plot_image(
         ax.set_facecolor(facecolor)
         plt.colorbar(im)
     return fig, ax
+
+
+def _combine_events(eventfiles, wcs_out, shape_out, outfile, overwrite=False):
+    x = []
+    y = []
+    e = []
+    ch = []
+    t = []
+    se = []
+    chantype = ""
+    header_dict = {}
+    for i, eventfile in enumerate(eventfiles):
+        with fits.open(eventfile, memmap=True) as f:
+            hdu = f["EVENTS"]
+            wcs_in = wcs_from_header(hdu.header)
+            if i == 0:
+                chantype = hdu.header["CHANTYPE"]
+                for key in [
+                    "TELESCOP",
+                    "INSTRUME",
+                    "ANCRFILE",
+                    "RESPFILE",
+                    "EXPOSURE",
+                    "CHANTYPE",
+                    "DATE",
+                    "DATE-OBS",
+                    "DATE-END",
+                    "MISSION",
+                    "TSTART",
+                    "TSTOP",
+                    "TLMIN4",
+                    "TLMAX4",
+                    "PHA_BINS",
+                ]:
+                    header_dict[key] = hdu.header[key]
+                idxs = ...
+            else:
+                for key in [
+                    "TELESCOP",
+                    "INSTRUME",
+                    "ANCRFILE",
+                    "RESPFILE",
+                    "MISSION",
+                    "TLMIN4",
+                    "TLMAX4",
+                    "PHA_BINS",
+                    "CHANTYPE",
+                ]:
+                    v1 = hdu.header[key]
+                    v2 = header_dict[key]
+                    if v1 != v2:
+                        raise ValueError(
+                            f"'{key}' from {eventfile} "
+                            f"does not match from {eventfiles[0]}! "
+                            f"{v1} != {v2}!"
+                        )
+                if hdu.header["EXPOSURE"] < header_dict["EXPOSURE"]:
+                    raise ValueError(
+                        f"Exposure time for {eventfile} is less than "
+                        f"that for {eventfiles[0]}! {hdu.header['EXPOSURE']} "
+                        f"< {header_dict['EXPOSURE']}!"
+                    )
+                idxs = hdu.data["TIME"] < header_dict["EXPOSURE"]
+
+            ra, dec = wcs_in.wcs_pix2world(hdu.data["X"][idxs], hdu.data["Y"][idxs], 1)
+            xx, yy = wcs_out.wcs_world2pix(ra, dec, 1)
+            x.append(xx)
+            y.append(yy)
+            e.append(hdu.data["ENERGY"][idxs])
+            se.append(hdu.data["SOXS_ENERGY"][idxs])
+            ch.append(hdu.data[hdu.header["CHANTYPE"]][idxs])
+            t.append(hdu.data["TIME"][idxs])
+
+    x = np.concatenate(x)
+    y = np.concatenate(y)
+    e = np.concatenate(e)
+    se = np.concatenate(se)
+    ch = np.concatenate(ch)
+    t = np.concatenate(t)
+
+    if chantype == "PHA":
+        cunit = "adu"
+    elif chantype == "PI":
+        cunit = "Chan"
+
+    col_x = fits.Column(name="X", format="D", unit="pixel", array=x)
+    col_y = fits.Column(name="Y", format="D", unit="pixel", array=y)
+    col_e = fits.Column(name="ENERGY", format="E", unit="eV", array=e)
+    col_ch = fits.Column(name=chantype, format="1J", unit=cunit, array=ch)
+    col_t = fits.Column(name="TIME", format="1D", unit="s", array=t)
+    col_se = fits.Column(name="SOXS_ENERGY", format="E", unit="eV", array=se)
+
+    coldefs = fits.ColDefs([col_e, col_x, col_y, col_ch, col_t, col_se])
+    tbhdu = fits.BinTableHDU.from_columns(coldefs)
+    tbhdu.name = "EVENTS"
+
+    tbhdu.header["HDUVERS"] = "1.1.0"
+    tbhdu.header["RADECSYS"] = "FK5"
+    tbhdu.header["EQUINOX"] = 2000.0
+    tbhdu.header["HDUCLASS"] = "OGIP"
+    tbhdu.header["HDUCLAS1"] = "EVENTS"
+    tbhdu.header["HDUCLAS2"] = "ACCEPTED"
+    tbhdu.header["MTYPE1"] = "sky"
+    tbhdu.header["MFORM1"] = "x,y"
+    tbhdu.header["MTYPE2"] = "EQPOS"
+    tbhdu.header["MFORM2"] = "RA,DEC"
+    tbhdu.header["TCTYP2"] = "RA---TAN"
+    tbhdu.header["TCTYP3"] = "DEC--TAN"
+    tbhdu.header["TCRVL2"] = wcs_out.wcs.crval[0]
+    tbhdu.header["TCRVL3"] = wcs_out.wcs.crval[1]
+    tbhdu.header["TCDLT2"] = wcs_out.wcs.cdelt[0]
+    tbhdu.header["TCDLT3"] = wcs_out.wcs.cdelt[1]
+    tbhdu.header["TCRPX2"] = wcs_out.wcs.crpix[0]
+    tbhdu.header["TCRPX3"] = wcs_out.wcs.crpix[1]
+    tbhdu.header["TCUNI2"] = "deg"
+    tbhdu.header["TCUNI3"] = "deg"
+    tbhdu.header["TLMIN2"] = 0.5
+    tbhdu.header["TLMIN3"] = 0.5
+    tbhdu.header["TLMAX2"] = shape_out[0] + 0.5
+    tbhdu.header["TLMAX3"] = shape_out[1] + 0.5
+    tbhdu.header["CHANTYPE"] = chantype
+
+    with fits.open(eventfiles[0], memmap=True) as f:
+        for key in [
+            "TELESCOP",
+            "INSTRUME",
+            "ANCRFILE",
+            "RESPFILE",
+            "EXPOSURE",
+            "DATE",
+            "DATE-OBS",
+            "DATE-END",
+            "MISSION",
+            "TSTART",
+            "TSTOP",
+            "TLMIN4",
+            "TLMAX4",
+            "PHA_BINS",
+        ]:
+            tbhdu.header[key] = f["EVENTS"].header[key]
+
+    start = fits.Column(name="START", format="1D", unit="s", array=np.array([0.0]))
+    stop = fits.Column(
+        name="STOP", format="1D", unit="s", array=np.array([tbhdu.header["EXPOSURE"]])
+    )
+
+    tbhdu_gti = fits.BinTableHDU.from_columns([start, stop])
+    tbhdu_gti.name = "STDGTI"
+    tbhdu_gti.header["TSTART"] = 0.0
+    tbhdu_gti.header["TSTOP"] = tbhdu.header["EXPOSURE"]
+    tbhdu_gti.header["HDUCLASS"] = "OGIP"
+    tbhdu_gti.header["HDUCLAS1"] = "GTI"
+    tbhdu_gti.header["HDUCLAS2"] = "STANDARD"
+    tbhdu_gti.header["RADECSYS"] = "FK5"
+    tbhdu_gti.header["EQUINOX"] = 2000.0
+    tbhdu_gti.header["DATE"] = tbhdu.header["DATE"]
+    tbhdu_gti.header["DATE-OBS"] = tbhdu.header["DATE-OBS"]
+    tbhdu_gti.header["DATE-END"] = tbhdu.header["DATE-END"]
+
+    hdulist = [fits.PrimaryHDU(), tbhdu, tbhdu_gti]
+
+    fits.HDUList(hdulist).writeto(outfile, overwrite=overwrite)
+
+
+def merge_event_files(input_files, output_file, overwrite=False):
+    """
+    Merge SOXS-generated event files together. The WCS used for the final
+    file will be based on the first source file, so the original celestial
+    coordinates for the other files will be discarded.
+
+    Parameters
+    ----------
+    input_files : list of strings
+        The events file(s) containing the background events.
+    output_file : string
+        The merged events file.
+    overwrite : boolean, optional
+        Whether to overwrite an existing file with
+        the same name. Default: False
+    """
+    with fits.open(input_files[0], memmap=True) as f:
+        wcs_out = wcs_from_header(f["EVENTS"].header)
+        shape_out = 2.0 * wcs_out.wcs.crpix - 1
+    _combine_events(input_files, wcs_out, shape_out, output_file, overwrite=overwrite)
