@@ -10,7 +10,7 @@ from soxs.events import write_event_file
 from soxs.instrument_registry import instrument_registry
 from soxs.psf import psf_model_registry
 from soxs.response import AuxiliaryResponseFile, RedistributionMatrixFile
-from soxs.simput import SimputPhotonList, read_simput_catalog
+from soxs.simput import SimputCatalog, SimputPhotonList, read_simput_catalog
 from soxs.utils import (
     create_region,
     ensure_numpy_array,
@@ -54,7 +54,7 @@ def make_source_list(source):
                 parameters["src_names"][i],
             )
             source_list.append(phlist)
-    elif isinstance(source, str):
+    elif isinstance(source, (str, SimputCatalog)):
         # Assume this is a SIMPUT catalog
         source_list, parameters = read_simput_catalog(source)
     return source_list, parameters
@@ -85,7 +85,7 @@ def generate_events(
 
     Parameters
     ----------
-    input_events : string, dict, or None
+    input_events : string, dict, SimputCatalog, or None
         The unconvolved events to be used as input. Can be one of the
         following:
         1. The name of a SIMPUT catalog file.
@@ -94,6 +94,7 @@ def generate_events(
         "dec": A NumPy array of declination values in degrees.
         "energy": A NumPy array of energy values in keV.
         "flux": The flux of the entire source, in units of erg/cm**2/s.
+        3. A SimputCatalog object.
     out_file : string
         The name of the event file to be written.
     exp_time : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`
@@ -349,9 +350,7 @@ def generate_events(
                 all_events[key] = np.concatenate([all_events[key], events[key]])
 
     if len(all_events["energy"]) == 0:
-        mylog.warning(
-            "No events from any of the sources in " "the catalog were detected!"
-        )
+        mylog.warning("No events from any of the sources in the catalog were detected!")
         for key in [
             "xpix",
             "ypix",
@@ -360,6 +359,7 @@ def generate_events(
             "time",
             "chip_id",
             event_params["channel_type"],
+            "soxs_energy",
         ]:
             all_events[key] = np.array([])
     else:
@@ -589,11 +589,11 @@ def make_background_file(
         Whether to overwrite an existing file with the same name.
         Default: False
     foreground : boolean, optional
-        Whether or not to include the Galactic foreground. Default: True
+        Whether to include the Galactic foreground. Default: True
     instr_bkgnd : boolean, optional
-        Whether or not to include the instrumental background. Default: True
+        Whether to include the instrumental background. Default: True
     ptsrc_bkgnd : boolean, optional
-        Whether or not to include the point-source background. Default: True
+        Whether to include the point-source background. Default: True
     no_dither : boolean, optional
         If True, turn off dithering entirely. Default: False
     dither_params : array-like of floats, optional
@@ -813,6 +813,7 @@ def simulate_spectrum(
     foreground=False,
     ptsrc_bkgnd=False,
     bkgnd_area=None,
+    noisy=True,
     overwrite=False,
     prng=None,
     **kwargs,
@@ -848,6 +849,10 @@ def simulate_spectrum(
         The area on the sky for the background components, in square arcminutes.
         Default: None, necessary to specify if any of the background components
         are turned on.
+    noisy : boolean, optional
+        If False, simulate_spectrum will not use counting (Poisson) statistics
+        when creating the spectrum. If any backgrounds are on, this must be
+        set to True. Default: True
     overwrite : boolean, optional
         Whether to overwrite an existing file. Default: False
     prng : :class:`~numpy.random.RandomState` object, integer, or None
@@ -873,6 +878,11 @@ def simulate_spectrum(
     from soxs.spectra import ConvolvedSpectrum
     from soxs.utils import soxs_cfg
 
+    if not noisy and any([instr_bkgnd, ptsrc_bkgnd, foreground]):
+        raise NotImplementedError(
+            "Backgrounds cannot be included in "
+            "simulations of non-noisy spectra at this time!"
+        )
     if "nH" in kwargs or "absorb_model" in kwargs:
         warnings.warn(
             "The 'nH' and 'absorb_model' keyword arguments"
@@ -920,7 +930,7 @@ def simulate_spectrum(
 
     if spec is not None:
         cspec = ConvolvedSpectrum.convolve(spec, arf, use_arf_energies=True)
-        out_spec += rmf.convolve_spectrum(cspec, exp_time, prng=prng)
+        out_spec += rmf.convolve_spectrum(cspec, exp_time, prng=prng, noisy=noisy)
 
     fov = None if bkgnd_area is None else np.sqrt(bkgnd_area)
 
@@ -963,6 +973,7 @@ def simulate_spectrum(
         event_params,
         out_file,
         overwrite=overwrite,
+        noisy=noisy,
     )
 
 
@@ -1045,10 +1056,8 @@ def simple_event_list(
                 all_events[key] = np.concatenate([all_events[key], events[key]])
 
     if len(all_events["energy"]) == 0:
-        mylog.warning(
-            "No events from any of the sources in " "the catalog were detected!"
-        )
-        for key in ["ra", "dec", "time", event_params["channel_type"]]:
+        mylog.warning("No events from any of the sources in the catalog were detected!")
+        for key in ["ra", "dec", "time", event_params["channel_type"], "soxs_energy"]:
             all_events[key] = np.array([])
     else:
         # Step 4: Scatter energies with RMF
@@ -1076,6 +1085,12 @@ def simple_event_list(
     col_e = fits.Column(
         name="ENERGY", format="E", unit="eV", array=all_events["energy"] * 1000.0
     )
+    col_se = fits.Column(
+        name="SOXS_ENERGY",
+        format="E",
+        unit="eV",
+        array=all_events["soxs_energy"] * 1000.0,
+    )
 
     chantype = event_params["channel_type"].upper()
     if chantype == "PHA":
@@ -1088,7 +1103,7 @@ def simple_event_list(
 
     col_t = fits.Column(name="TIME", format="1D", unit="s", array=all_events["time"])
 
-    cols = [col_e, col_ra, col_dec, col_ch, col_t]
+    cols = [col_e, col_ra, col_dec, col_ch, col_t, col_se]
 
     coldefs = fits.ColDefs(cols)
     tbhdu = fits.BinTableHDU.from_columns(coldefs)
