@@ -2,7 +2,7 @@ import numpy as np
 from astropy.io import ascii, fits
 from astropy.table import Table
 
-from soxs.events import make_exposure_map, wcs_from_header, write_image
+from soxs.events import _combine_events, make_exposure_map, write_image
 from soxs.instrument import instrument_simulator
 from soxs.utils import mylog
 
@@ -118,115 +118,6 @@ def make_mosaic_events(
     return outfile
 
 
-def _combine_events(eventfiles, wcs_out, shape_out, outfile, overwrite=False):
-    x = []
-    y = []
-    e = []
-    ch = []
-    t = []
-    chantype = ""
-    for eventfile in eventfiles:
-        with fits.open(eventfile, memmap=True) as f:
-            hdu = f["EVENTS"]
-            wcs_in = wcs_from_header(hdu.header)
-            ra, dec = wcs_in.wcs_pix2world(hdu.data["X"], hdu.data["Y"], 1)
-            xx, yy = wcs_out.wcs_world2pix(ra, dec, 1)
-            x.append(xx)
-            y.append(yy)
-            e.append(hdu.data["ENERGY"])
-            ch.append(hdu.data[hdu.header["CHANTYPE"]])
-            t.append(hdu.data["TIME"])
-            chantype = hdu.header["CHANTYPE"]
-
-    x = np.concatenate(x)
-    y = np.concatenate(y)
-    e = np.concatenate(e)
-    ch = np.concatenate(ch)
-    t = np.concatenate(t)
-
-    if chantype == "PHA":
-        cunit = "adu"
-    elif chantype == "PI":
-        cunit = "Chan"
-
-    col_x = fits.Column(name="X", format="D", unit="pixel", array=x)
-    col_y = fits.Column(name="Y", format="D", unit="pixel", array=y)
-    col_e = fits.Column(name="ENERGY", format="E", unit="eV", array=e)
-    col_ch = fits.Column(name=chantype, format="1J", unit=cunit, array=ch)
-    col_t = fits.Column(name="TIME", format="1D", unit="s", array=t)
-
-    coldefs = fits.ColDefs([col_e, col_x, col_y, col_ch, col_t])
-    tbhdu = fits.BinTableHDU.from_columns(coldefs)
-    tbhdu.name = "EVENTS"
-
-    tbhdu.header["HDUVERS"] = "1.1.0"
-    tbhdu.header["RADECSYS"] = "FK5"
-    tbhdu.header["EQUINOX"] = 2000.0
-    tbhdu.header["HDUCLASS"] = "OGIP"
-    tbhdu.header["HDUCLAS1"] = "EVENTS"
-    tbhdu.header["HDUCLAS2"] = "ACCEPTED"
-    tbhdu.header["MTYPE1"] = "sky"
-    tbhdu.header["MFORM1"] = "x,y"
-    tbhdu.header["MTYPE2"] = "EQPOS"
-    tbhdu.header["MFORM2"] = "RA,DEC"
-    tbhdu.header["TCTYP2"] = "RA---TAN"
-    tbhdu.header["TCTYP3"] = "DEC--TAN"
-    tbhdu.header["TCRVL2"] = wcs_out.wcs.crval[0]
-    tbhdu.header["TCRVL3"] = wcs_out.wcs.crval[1]
-    tbhdu.header["TCDLT2"] = wcs_out.wcs.cdelt[0]
-    tbhdu.header["TCDLT3"] = wcs_out.wcs.cdelt[1]
-    tbhdu.header["TCRPX2"] = wcs_out.wcs.crpix[0]
-    tbhdu.header["TCRPX3"] = wcs_out.wcs.crpix[1]
-    tbhdu.header["TCUNI2"] = "deg"
-    tbhdu.header["TCUNI3"] = "deg"
-    tbhdu.header["TLMIN2"] = 0.5
-    tbhdu.header["TLMIN3"] = 0.5
-    tbhdu.header["TLMAX2"] = shape_out[0] + 0.5
-    tbhdu.header["TLMAX3"] = shape_out[1] + 0.5
-    tbhdu.header["CHANTYPE"] = chantype
-
-    with fits.open(eventfiles[0], memmap=True) as f:
-        for key in [
-            "TELESCOP",
-            "INSTRUME",
-            "ANCRFILE",
-            "RESPFILE",
-            "EXPOSURE",
-            "DATE",
-            "DATE-OBS",
-            "DATE-END",
-            "MISSION",
-            "TSTART",
-            "TSTOP",
-            "TLMIN4",
-            "TLMAX4",
-            "PHA_BINS",
-        ]:
-            tbhdu.header[key] = f["EVENTS"].header[key]
-
-    start = fits.Column(name="START", format="1D", unit="s", array=np.array([0.0]))
-    stop = fits.Column(
-        name="STOP", format="1D", unit="s", array=np.array([tbhdu.header["EXPOSURE"]])
-    )
-
-    tbhdu_gti = fits.BinTableHDU.from_columns([start, stop])
-    tbhdu_gti.name = "STDGTI"
-    tbhdu_gti.header["TSTART"] = 0.0
-    tbhdu_gti.header["TSTOP"] = tbhdu.header["EXPOSURE"]
-    tbhdu_gti.header["HDUCLASS"] = "OGIP"
-    tbhdu_gti.header["HDUCLAS1"] = "GTI"
-    tbhdu_gti.header["HDUCLAS2"] = "STANDARD"
-    tbhdu_gti.header["RADECSYS"] = "FK5"
-    tbhdu_gti.header["EQUINOX"] = 2000.0
-    tbhdu_gti.header["DATE"] = tbhdu.header["DATE"]
-    tbhdu_gti.header["DATE-OBS"] = tbhdu.header["DATE-OBS"]
-    tbhdu_gti.header["DATE-END"] = tbhdu.header["DATE-END"]
-
-    hdulist = [fits.PrimaryHDU(), tbhdu, tbhdu_gti]
-
-    fits.HDUList(hdulist).writeto(outfile, overwrite=overwrite)
-
-
 def make_mosaic_image(
     evtfile_list,
     image_file,
@@ -267,7 +158,7 @@ def make_mosaic_image(
         Supply an integer power of 2 here to make an exposure map
         with a different binning. Default: 1
     use_expmap : boolean, optional
-        Whether or not to use (and potentially generate) an exposure map
+        Whether to use (and potentially generate) an exposure map
         and a flux map. Default: False
     expmap_energy : float, (value, unit) tuple, or :class:`~astropy.units.Quantity`, or NumPy array, optional
         The energy in keV to use when computing the exposure map, or
