@@ -8,6 +8,7 @@ from pathlib import Path
 
 from soxs.spatial import construct_wcs
 from soxs.utils import (
+    convert_endian,
     ensure_numpy_array,
     mylog,
     parse_prng,
@@ -66,7 +67,10 @@ def read_simput_catalog(simput_file):
        and energies of the events from the sources.
     2. NumPy arrays of the parameters of the sources.
     """
-    sc = SimputCatalog.from_file(simput_file)
+    if isinstance(simput_file, SimputCatalog):
+        sc = simput_file
+    else:
+        sc = SimputCatalog.from_file(simput_file)
     parameters = {
         "emin": sc.emin,
         "emax": sc.emax,
@@ -139,6 +143,8 @@ class SimputCatalog:
             The name of the SIMPUT catalog file to read the
             catalog and photon lists from.
         """
+        from astropy.io.fits.column import _VLF
+
         f_simput = fits.open(filename)
         fluxes = f_simput["src_cat"].data["flux"]
         src_names = f_simput["src_cat"].data["src_name"]
@@ -147,8 +153,12 @@ class SimputCatalog:
         ra = f_simput["src_cat"].data["ra"]
         dec = f_simput["src_cat"].data["dec"]
         spectra = f_simput["src_cat"].data["spectrum"]
+        if isinstance(spectra, _VLF):
+            spectra = [s.astype("|S1").tobytes().decode("utf-8") for s in spectra]
         if "IMAGE" in f_simput["src_cat"].columns.names:
             images = f_simput["src_cat"].data["image"]
+            if isinstance(images, _VLF):
+                images = [i.astype("|S1").tobytes().decode("utf-8") for i in images]
         else:
             images = ["NULL"] * len(spectra)
         f_simput.close()
@@ -175,15 +185,15 @@ class SimputCatalog:
         ext = extname if extver is None else (extname, extver)
         data = fits.getdata(fn_src, ext)
         if extname == "phlist":
-            ra = Quantity(data["ra"], "deg")
-            dec = Quantity(data["dec"], "deg")
-            energy = Quantity(data["energy"], "keV")
+            ra = Quantity(convert_endian(data["ra"]).astype("float64"), "deg")
+            dec = Quantity(convert_endian(data["dec"]).astype("float64"), "deg")
+            energy = Quantity(convert_endian(data["energy"]).astype("float64"), "keV")
             src = SimputPhotonList(
                 ra, dec, energy, self.fluxes[i], name=self.src_names[i]
             )
         elif extname == "spectrum":
-            emid = data["energy"]
-            flux = data["fluxdensity"]
+            emid = convert_endian(data["energy"])
+            flux = convert_endian(data["fluxdensity"])
             if isinstance(data["energy"], _VLF):
                 emid = emid[0]
                 flux = flux[0]
