@@ -1051,11 +1051,16 @@ class ConvolvedSpectrum(CountRateSpectrum):
         if not isinstance(arf, AuxiliaryResponseFile):
             arf = AuxiliaryResponseFile(arf)
         if use_arf_energies:
-            flux = u.Quantity(
-                np.interp(
-                    arf.emid, spectrum.emid.value, spectrum.flux, left=0.0, right=0.0
-                ),
-                "ph/s/cm**2/keV",
+            flux = (
+                u.Quantity(
+                    regrid_spectrum(
+                        arf.ebins,
+                        spectrum.ebins.value,
+                        spectrum.flux.value * spectrum.de.value,
+                    ),
+                    "cm-2 keV-1 ph s-1",
+                )
+                / arf.de
             )
             rate = u.Quantity(arf.eff_area, "cm**2") * flux
             binscale = "linear"
