@@ -5,13 +5,14 @@ if [[ ${mode} == "testing" ]]; then
   # Download answers
 
   curl -O http://hea-www.cfa.harvard.edu/~jzuhone/soxs_test_data_${ANSWER_VER}.tar.gz
-  tar -zxf soxs_test_data_${ANSWER_VER}.tar.gz
+  tar -zxvf soxs_test_data_${ANSWER_VER}.tar.gz
 
   # Set location of soxs data
 
   mkdir -p $HOME/.config/soxs
   echo "[soxs]" > $HOME/.config/soxs/soxs.cfg
-  echo "soxs_data_dir = \"${PWD}/soxs_data\"" >> $HOME/.config/soxs/soxs.cfg
+  echo "soxs_data_dir = ${GITHUB_WORKSPACE}/soxs_data" >> $HOME/.config/soxs/soxs.cfg
+  echo "soxs_answer_dir = ${GITHUB_WORKSPACE}/soxs_test_data" >> $HOME/.config/soxs/soxs.cfg
   cat $HOME/.config/soxs/soxs.cfg
 
 fi
